==> nodus-framework-0.0.31.tgz: package/lib/activities.js <==
'use strict';

// ** Dependencies
const _ = require('lodash');
const util = require('util');
const functions = require('./functions');
const errors = require('./errors');
const Promise = require('bluebird');
const logger = require('./logging').createLogger('activities');
const EventEmitter = require('./events').EventEmitter;
const microtime = require('microtime');

/**
 * Inspect the function signature to build a list of parameters
 * @param func
 */
function getParameters(func) {

    const parameterNames = functions.getParameterNames(func);
    logger.debug('PARAMETER_NAMES', parameterNames);

    const ret = {};

    _.forEach(parameterNames, name => {
        ret[name] = {
            defaultValue: undefined,
            required: false
        };
    });

    return ret;
}

function setParameters(parameters, args) {

    const ret = {};

    // Set the value of each parameter from the args, or use defaults
    _.forEach(parameters, (param, name) => {
        // Use value from args or apply a default value
        const value = args.hasOwnProperty(name)
            ? args[name]
            : param.defaultValue;

        logger.debug('SET_PARAMETER', {
            name: name,
            value: value
        });

        ret[name] = value;
    });

    return ret;
}

function buildContext(context, params, options) {

    if (util.isNullOrUndefined(context))
        context = {};

    context['__params'] = params;
    context['__options'] = options;

    return context;
}

class Activity extends EventEmitter {
    constructor(name, handler, options) {
        super();

        this._name = name;
        this._handler = handler;
        this._parameters = options.parameters || getParameters(handler);

        // Set run function so we can capture 'this' from the caller
        const self = this;
        this.run = function (args, options) {

            // Default options (for the run command)
            options = options || {};

            // Extract 'completed' handler from the activity
            const completed = self._completed;

            // Start the activity
            const started_on = microtime.now();

            // Function to track how much time has elapsed since the activity was started
            const TIMESTAMP = () => {
                const ts = microtime.now();

                return {
                    timestamp: ts,
                    started_at: started_on,
                    elapsed_ms: (ts - started_on) * .001
                };
            };

            // Emit events with timestamps
            const EMIT = (event, data) => self.emit(event, {
                timestamp: TIMESTAMP(),
                data: data,
            });

            // Emit that this instance is running
            EMIT('run', {args: args, options: options});

            return new Promise((resolve, reject) => {
                try {
                    // Build an array of arguments to pass to the caller
                    const params = setParameters(self._parameters, args);
                    logger.debug('PARAMS', params);

                    // Create context for the activity
                    const context = buildContext(this, params, options);
                    logger.debug('CONTEXT', context);

                    // Convert parameters to array to apply to function
                    const argList = _.values(params);
                    logger.debug('ARG_LIST', argList);

                    // Send 'run' notification event
                    EMIT('exec', {
                        params: params,
                        options: options
                    });

                    // Invoke the function with the specified context
                    const result = self._handler.apply(context, argList);

                    // Send 'result' event notification
                    EMIT('success', result);

                    resolve(result);
                } catch (err) {
                    // Send 'failure' event notification
                    EMIT('failure', err);

                    reject(err);
                }
            })
                .finally(() => {

                    // Run onComplete event if specified in the options
                    if (self._completed) {
                        logger.debug('***** Inside Complete Handler *****');
                        return Promise
                            .resolve(self._completed())
                            .finally(() => EMIT('completed'))
                    }

                    // No complete handler, just emit the 'completed' event
                    EMIT('completed');
                });
        };
    }

    get name() {
        return this._name;
    }

    set completed(handler) {
        this._completed = handler;
    }
}

function isActivity(obj) {
    return obj instanceof Activity;
}

function createActivity(name, handler, options) {
    logger.debug('CREATE_ACTIVITY', arguments);

    // createCommand(function) -> createCommand(function.name, function)
    if (arguments.length === 1 && util.isFunction(arguments[0]))
        return createActivity(arguments[0].name, arguments[0]);

    // createCommand({name:, handler:, ...} -> createCommand(.name, .handler, {...})
    if (arguments.length === 1 && util.isObject(arguments[0]))
        return createActivity(arguments[0].name, arguments[0].handler, arguments[0]);

    // Argument Defaults
    options = options || {};

    // Name is a required argument
    if (_.isEmpty(name))
        throw errors('ARGUMENT_REQUIRED', 'The argument "name" is required.', {name: name});

    // Handler must be a function
    if (!util.isFunction(handler))
        throw errors('ARGUMENT_INVALID', 'The argument "handler" must be a function.', {handler: handler});

    logger.debug('CREATE_ACTIVITY', {
        name: name,
        handler: handler,
        options: options
    });

    const activity = new Activity(name, handler, options);
    return activity;
}

// ** Exports
module.exports = createActivity;
module.exports.isActivity = isActivity;
module.exports.create = createActivity;

==> nodus-framework-0.5.9.tgz: package/lib/activities.js <==
'use strict';

// ** Dependencies
const _ = require('lodash');
const util = require('util');
const functions = require('./functions');
const errors = require('./errors');
const Promise = require('bluebird');
const logger = require('./logger').createLogger('activities');
const EventEmitter = require('./events').EventEmitter;
const microtime = require('microtime');

/**
 * Inspect the function signature to build a list of parameters
 * @param func
 */
function getParameters(func) {

    const parameterNames = functions.getParameterNames(func);
    logger.debug('PARAMETER_NAMES', parameterNames);

    const ret = {};

    _.forEach(parameterNames, name => {
        ret[name] = {
            defaultValue: undefined,
            required: false
        };
    });

    return ret;
}

function setParameters(parameters, args) {

    const ret = {};

    // Set the value of each parameter from the args, or use defaults
    _.forEach(parameters, (param, name) => {
        // Use value from args or apply a default value
        const value = args.hasOwnProperty(name)
            ? args[name]
            : param.defaultValue;

        logger.debug('SET_PARAMETER', {
            name: name,
            value: value
        });

        ret[name] = value;
    });

    return ret;
}

function buildContext(context, params, options) {

    if (util.isNullOrUndefined(context))
        context = {};

    context['__params'] = params;
    context['__options'] = options;

    return context;
}

class Activity extends EventEmitter {
    constructor(name, handler, options) {
        super();

        this._name = name;
        this._handler = handler;
        this._parameters = options.parameters || getParameters(handler);

        // Set run function so we can capture 'this' from the caller
        const self = this;
        this.run = function (args, options) {

            // Default options (for the run command)
            options = options || {};

            // Extract 'completed' handler from the activity
            const completed = self._completed;

            // Start the activity
            const started_on = microtime.now();

            // Function to track how much time has elapsed since the activity was started
            const TIMESTAMP = () => {
                const ts = microtime.now();

                return {
                    timestamp: ts,
                    started_at: started_on,
                    elapsed_ms: (ts - started_on) * .001
                };
            };

            // Emit events with timestamps
            const EMIT = (event, data) => self.emit(event, {
                timestamp: TIMESTAMP(),
                data: data,
            });

            // Emit that this instance is running
            EMIT('run', {args: args, options: options});

            return new Promise((resolve, reject) => {
                try {
                    // Build an array of arguments to pass to the caller
                    const params = setParameters(self._parameters, args);
                    logger.debug('PARAMS', params);

                    // Create context for the activity
                    const context = buildContext(this, params, options);
                    logger.debug('CONTEXT', context);

                    // Convert parameters to array to apply to function
                    const argList = _.values(params);
                    logger.debug('ARG_LIST', argList);

                    // Send 'run' notification event
                    EMIT('exec', {
                        params: params,
                        options: options
                    });

                    // Invoke the function with the specified context
                    const result = self._handler.apply(context, argList);

                    // Send 'result' event notification
                    EMIT('success', result);

                    resolve(result);
                } catch (err) {
                    // Send 'failure' event notification
                    EMIT('failure', err);

                    reject(err);
                }
            })
                .finally(() => {

                    // Run onComplete event if specified in the options
                    if (self._completed) {
                        logger.debug('***** Inside Complete Handler *****');
                        return Promise
                            .resolve(self._completed())
                            .finally(() => EMIT('completed'))
                    }

                    // No complete handler, just emit the 'completed' event
                    EMIT('completed');
                });
        };
    }

    get name() {
        return this._name;
    }

    set completed(handler) {
        this._completed = handler;
    }
}

function isActivity(obj) {
    return obj instanceof Activity;
}

function createActivity(name, handler, options) {
    logger.debug('CREATE_ACTIVITY', arguments);

    // createCommand(function) -> createCommand(function.name, function)
    if (arguments.length === 1 && util.isFunction(arguments[0]))
        return createActivity(arguments[0].name, arguments[0]);

    // createCommand({name:, handler:, ...} -> createCommand(.name, .handler, {...})
    if (arguments.length === 1 && util.isObject(arguments[0]))
        return createActivity(arguments[0].name, arguments[0].handler, arguments[0]);

    // Argument Defaults
    options = options || {};

    // Name is a required argument
    if (_.isEmpty(name))
        throw errors('ARGUMENT_REQUIRED', 'The argument "name" is required.', {name: name});

    // Handler must be a function
    if (!util.isFunction(handler))
        throw errors('ARGUMENT_INVALID', 'The argument "handler" must be a function.', {handler: handler});

    logger.debug('CREATE_ACTIVITY', {
        name: name,
        handler: handler,
        options: options
    });

    const activity = new Activity(name, handler, options);
    return activity;
}

// ** Exports
module.exports = createActivity;
module.exports.isActivity = isActivity;
module.exports.create = createActivity;
